<?php

/**
 * @file
 * Contains SearchApiAlterFileEntityPublic.
 */

/**
 * Exclude file entity not in the public folder from the indexes.
 */
class SearchApiAlterFileEntityPublic extends SearchApiAbstractAlterCallback {

  /**
   * Check whether this data-alter callback is applicable for a certain index.
   *
   * Returns TRUE only for indexes on file entity.
   *
   * @param SearchApiIndex $index
   *   The index to check for.
   *
   * @return boolean
   *   TRUE if the callback can run on the given index; FALSE otherwise.
   */
  public function supportsIndex(SearchApiIndex $index) {
    if ($this->isMultiEntityIndex($index)) {
      return in_array('file', $index->options['datasource']['types']);
    }
    return $index->getEntityType() === 'file';
  }

  /**
   * Alter items before indexing.
   *
   * Items which are removed from the array won't be indexed, but will be marked
   * as clean for future indexing.
   *
   * @param array $items
   *   An array of items to be altered, keyed by item IDs.
   */
  public function alterItems(array &$items) {
    $multi_types = $this->isMultiEntityIndex($this->index);
    foreach ($items as $id => $item) {
      $file = $item;
      if ($multi_types) {
        if ($item->item_type !== 'file') {
          continue;
        }
        $file = $item->file;
      }
      if (empty($file->uri) || strpos($file->uri, 'private://') !== FALSE) {
        unset($items[$id]);
      }
    }
  }

}
